@@ -51,7 +51,7 @@ extern char *getenv();
 #include <cairo/cairo-xlib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#ifdef __APPLE__
+#ifdef OSXAPP
 #  include <gtkmacintegration/gtkosxapplication.h>
 #endif
 
@@ -138,16 +138,6 @@ static Arg formArgs[] = {
 };
 #endif
 
-void
-MarkMenuItem (char *menuRef, int state)
-{
-    MenuItem *item = MenuNameToItem(menuRef);
-
-    if(item) {
-        ((GtkCheckMenuItem *) (item->handle))->active = state;
-    }
-}
-
 void GetWidgetTextGTK(GtkWidget *w, char **buf)
 {
     GtkTextIter start;
@@ -259,6 +249,15 @@ SetDialogTitle (DialogClass dlg, char *title)
     gtk_window_set_title(GTK_WINDOW(shells[dlg]), title);
 }
 
+void
+SetWidgetFont (GtkWidget *w, char **s)
+{
+    PangoFontDescription *pfd;
+    if (!s || !*s || !**s) return; // uses no font, no font spec or empty font spec
+    pfd = pango_font_description_from_string(*s);
+    gtk_widget_modify_font(w, pfd);
+}
+
 void
 SetListBoxItem (GtkListStore *store, int n, char *msg)
 {
@@ -320,13 +319,10 @@ void
 ScrollToCursor (Option *opt, int caretPos)
 {
     static GtkTextIter iter;
-    static GtkTextMark *mark;
-    if(!mark) mark = gtk_text_mark_new(NULL, 0);
+    GtkTextMark *mark = gtk_text_buffer_get_mark((GtkTextBuffer *) opt->handle, "scrollmark");
     gtk_text_buffer_get_iter_at_offset((GtkTextBuffer *) opt->handle, &iter, caretPos);
-    gtk_text_buffer_add_mark((GtkTextBuffer *) opt->handle, mark, &iter);
+    gtk_text_buffer_move_mark((GtkTextBuffer *) opt->handle, mark, &iter);
     gtk_text_view_scroll_to_mark((GtkTextView *) opt->textValue, mark, 0.0, 0, 0.5, 0.5);
-//    gtk_text_view_scroll_to_iter((GtkTextView *) opt->textValue, &iter, 0.0, 0, 0.5, 0.5);
-    gtk_text_buffer_delete_mark((GtkTextBuffer *) opt->handle, mark);
 }
 
 int
@@ -353,6 +349,7 @@ FocusOnWidget (Option *opt, DialogClass dlg)
 #ifdef TODO_GTK
     XtSetKeyboardFocus(shells[dlg], opt->handle);
 #endif
+    if(dlg) gtk_window_present(GTK_WINDOW(shells[dlg]));
     gtk_widget_grab_focus(opt->handle);
 }
 
@@ -428,10 +425,11 @@ CreateMenuPopup (Option *opt, int n, int def)
       {
 	char *msg = mb[i].string;
 	if(!msg) break;
-#ifdef __APPLE__
+#ifdef OSXAPP
 	if(!strcmp(msg, "Quit ")) continue;             // Quit item will appear automatically in App menu
 	if(!strcmp(msg, "About XBoard")) msg = "About"; // 'XBoard' will be appended automatically when moved to App menu 1st item
 #endif
+        if(!strcmp(msg, "ICS Input Box")) { mb[i].handle = NULL; continue; } // suppress ICS Input Box in GTK
 	if(strcmp(msg, "----")) { //
 	  if(!(opt->min & NO_GETTEXT)) msg = _(msg);
 	  if(mb[i].handle) {
@@ -445,11 +443,11 @@ CreateMenuPopup (Option *opt, int n, int def)
 	    GdkModifierType accelerator_mods;
 
 	    gtk_accelerator_parse(mb[i].accel, &accelerator_key, &accelerator_mods);
-#ifdef __APPLE__
+#ifdef OSXAPP
    	    if(accelerator_mods & GDK_CONTROL_MASK) {  // in OSX use Meta where Linux uses Ctrl
 		accelerator_mods &= ~GDK_CONTROL_MASK; // clear Ctrl flag
 		accelerator_mods |= GDK_META_MASK;     // set Meta flag
-	    } 
+	    }
 #endif
 	    gtk_widget_add_accelerator (GTK_WIDGET(entry), "activate",GtkAccelerators,
 					accelerator_key, accelerator_mods, GTK_ACCEL_VISIBLE);
@@ -504,11 +502,16 @@ TypeInProc (GtkWidget *widget, GdkEventKey *event, gpointer gdata)
     shiftState = event->state & GDK_SHIFT_MASK;
     controlState = event->state & GDK_CONTROL_MASK;
     switch(event->keyval) {
+      case 'n':       return (controlState && IcsHist(14, opt, dlg));
+      case 'o':       return (controlState && IcsHist(15, opt, dlg));
+      case GDK_Tab:   IcsHist(10, opt, dlg); break;
+      case GDK_Up:     IcsHist(1, opt, dlg); break;
+      case GDK_Down:  IcsHist(-1, opt, dlg); break;
       case GDK_Return:
 	if(GenericReadout(dialogOptions[dlg], -1)) PopDown(dlg);
 	break;
       case GDK_Escape:
-	PopDown(dlg);
+	if(!IcsHist(33, opt, dlg)) PopDown(dlg);
 	break;
       default:
 	return FALSE;
@@ -532,6 +535,61 @@ HighlightText (Option *opt, int from, int to, Boolean highlight)
     gtk_text_buffer_apply_tag_by_name(opt->handle, highlight ? "highlight" : "normal", &start, &end);
 }
 
+static char **names;
+static int curFG, curBG, curAttr;
+static GdkColor backgroundColor;
+
+void
+SetTextColor(char **cnames, int fg, int bg, int attr)
+{
+    if(fg < 0) fg = 0; if(bg < 0) bg = 7;
+    names = cnames; curFG = fg; curBG = bg, curAttr = attr;
+    if(attr == -2) { // background color of ICS console.
+	gdk_color_parse(cnames[bg&7], &backgroundColor);
+	curAttr = 0;
+    }
+}
+
+void
+AppendColorized (Option *opt, char *s, int count)
+{
+    static GtkTextIter end;
+    static GtkTextTag *fgTags[8], *bgTags[8], *font, *bold, *normal, *attr = NULL;
+
+    if(!font) {
+	font = gtk_text_buffer_create_tag(opt->handle, NULL, "font", appData.icsFont, NULL);
+	gtk_widget_modify_base(GTK_WIDGET(opt->textValue), GTK_STATE_NORMAL, &backgroundColor);
+    }
+
+    gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(opt->handle), &end);
+
+    if(names) {
+      if(curAttr == 1) {
+	if(!bold) bold = gtk_text_buffer_create_tag(opt->handle, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
+        attr = bold;
+      } else {
+	if(!normal) normal = gtk_text_buffer_create_tag(opt->handle, NULL, "weight", PANGO_WEIGHT_NORMAL, NULL);
+        attr = normal;
+      }
+      if(!fgTags[curFG]) {
+	fgTags[curFG] = gtk_text_buffer_create_tag(opt->handle, NULL, "foreground", names[curFG], NULL);
+      }
+      if(!bgTags[curBG]) {
+	bgTags[curBG] = gtk_text_buffer_create_tag(opt->handle, NULL, "background", names[curBG], NULL);
+      }
+      gtk_text_buffer_insert_with_tags(opt->handle, &end, s, count, fgTags[curFG], bgTags[curBG], font, attr, NULL);
+    } else
+      gtk_text_buffer_insert_with_tags(opt->handle, &end, s, count, font, NULL);
+
+}
+
+void
+Show (Option *opt, int hide)
+{
+    if(hide) gtk_widget_hide(opt->handle);
+    else     gtk_widget_show(opt->handle);
+}
+
 int
 ShiftKeys ()
 {   // bassic primitive for determining if modifier keys are pressed
@@ -602,15 +660,23 @@ MemoEvent(GtkWidget *widget, GdkEvent *event, gpointer gdata)
 		((ButtonCallback*) memo->target)(button == 1 ? memo->value : -memo->value);
 		return TRUE;
 	    }
+	    if(memo->value == 250 // kludge to recognize ICS Console and Chat panes
+	     && gtk_text_buffer_get_selection_bounds(memo->handle, NULL, NULL) ) {
+printf("*** selected\n");
+	        gtk_text_buffer_get_selection_bounds(memo->handle, &start, &end); // only return selected text
+		index = -1; // kludge to indicate omething was selected
+	    } else {
 // GTK_TODO: is this really the most efficient way to get the character at the mouse cursor???
-	    gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(widget), GTK_TEXT_WINDOW_WIDGET, w, h, &x, &y);
-	    gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(widget), &start, x, y);
-	    gtk_text_buffer_place_cursor(memo->handle, &start);
-	    /* get cursor position into index */
-	    g_object_get(memo->handle, "cursor-position", &index, NULL);
+		gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(widget), GTK_TEXT_WINDOW_WIDGET, w, h, &x, &y);
+		gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(widget), &start, x, y);
+		gtk_text_buffer_place_cursor(memo->handle, &start);
+		/* get cursor position into index */
+		g_object_get(memo->handle, "cursor-position", &index, NULL);
+		/* take complete contents */
+		gtk_text_buffer_get_start_iter (memo->handle, &start);
+		gtk_text_buffer_get_end_iter (memo->handle, &end);
+	    }
 	    /* get text from textbuffer */
-	    gtk_text_buffer_get_start_iter (memo->handle, &start);
-	    gtk_text_buffer_get_end_iter (memo->handle, &end);
 	    val = gtk_text_buffer_get_text (memo->handle, &start, &end, FALSE);
 	    break;
 	default:
@@ -652,7 +718,7 @@ AddHandler (Option *opt, DialogClass dlg, int nr)
 GtkWidget *shells[NrOfDialogs];
 DialogClass parents[NrOfDialogs];
 WindowPlacement *wp[NrOfDialogs] = { // Beware! Order must correspond to DialogClass enum
-    NULL, &wpComment, &wpTags, NULL, NULL, NULL, &wpDualBoard, &wpMoveHistory, &wpGameList, &wpEngineOutput, &wpEvalGraph,
+    NULL, &wpComment, &wpTags, &wpTextMenu, NULL, &wpConsole, &wpDualBoard, &wpMoveHistory, &wpGameList, &wpEngineOutput, &wpEvalGraph,
     NULL, NULL, NULL, NULL, &wpMain
 };
 
@@ -739,7 +805,8 @@ gboolean GenericPopDown(w, resptype, gdata)
 // I guess BrowserDlg will be abandoned, as GTK has a better browser of its own
     if(shellUp[BrowserDlg] && dlg != BrowserDlg || dialogError) return True; // prevent closing dialog when it has an open file-browse daughter
 #else
-    if(browserUp || dialogError && dlg != FatalDlg) return True; // prevent closing dialog when it has an open file-browse or error-popup daughter
+    if(browserUp || dialogError && dlg != FatalDlg || dlg == MasterDlg && shellUp[TransientDlg])
+	return True; // prevent closing dialog when it has an open file-browse, transient or error-popup daughter
 #endif
     shells[dlg] = w; // make sure we pop down the right one in case of multiple instances
 
@@ -802,6 +869,7 @@ GraphEventProc(GtkWidget *widget, GdkEvent *event, gpointer gdata)
     GdkEventExpose *eevent = (GdkEventExpose *) event;
     GdkEventButton *bevent = (GdkEventButton *) event;
     GdkEventMotion *mevent = (GdkEventMotion *) event;
+    GdkEventScroll *sevent = (GdkEventScroll *) event;
     GtkAllocation a;
     cairo_t *cr;
 
@@ -858,6 +926,10 @@ GraphEventProc(GtkWidget *widget, GdkEvent *event, gpointer gdata)
 	    cairo_destroy(cr);
 	default:
 	    return;
+	case GDK_SCROLL:
+	    if(sevent->direction == GDK_SCROLL_UP) button = 4;
+	    if(sevent->direction == GDK_SCROLL_DOWN) button = 5;
+	    break;
 	case GDK_MOTION_NOTIFY:
 	    f = 0;
 	    w = mevent->x; h = mevent->y;
@@ -1129,6 +1201,7 @@ GenericPopUp (Option *option, char *title, DialogClass dlgNr, DialogClass parent
 
     int i, j, arraysize, left, top, height=999, width=1, boxStart=0, breakType = 0, r;
     char def[MSG_SIZ], *msg, engineDlg = (currentCps != NULL && dlgNr != BrowserDlg);
+    gboolean expandable = FALSE;
 
     if(dlgNr < PromoDlg && shellUp[dlgNr]) return 0; // already up
 
@@ -1138,6 +1211,7 @@ GenericPopUp (Option *option, char *title, DialogClass dlgNr, DialogClass parent
 	if(wp[dlgNr]) gtk_window_move(GTK_WINDOW(shells[dlgNr]), wp[dlgNr]->x, wp[dlgNr]->y);
         return 0;
     }
+    if(dlgNr == TransientDlg && parent == BoardWindow && shellUp[MasterDlg]) parent = MasterDlg; // MasterDlg can always take role of main window
 
     dialogOptions[dlgNr] = option; // make available to callback
     // post currentOption globally, so Spin and Combo callbacks can already use it
@@ -1196,22 +1270,22 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
     top = -1;
 
     for (i=0;option[i].type != EndMark;i++) {
-	if(option[i].type == -1) continue;
+	if(option[i].type == Skip) continue;
         top++;
 //printf("option =%2d, top =%2d\n", i, top);
-        if (top >= height) {
-            gtk_table_resize(GTK_TABLE(table), height, r);
+        if (top >= height || breakType) {
+            gtk_table_resize(GTK_TABLE(table), top - (breakType != 0), r);
 	    if(!pane) { // multi-column: put tables in intermediate hbox
-		if(breakType || engineDlg)
+		if(breakType & SAME_ROW || engineDlg)
 		    pane =  gtk_hbox_new (FALSE, 0);
 		else
 		    pane =  gtk_vbox_new (FALSE, 0);
 		gtk_box_set_spacing(GTK_BOX(pane), 5 + 5*breakType);
 		gtk_box_pack_start (GTK_BOX (/*GTK_DIALOG (dialog)->vbox*/box), pane, TRUE, TRUE, 0);
 	    }
-	    gtk_box_pack_start (GTK_BOX (pane), table, TRUE, TRUE, 0);
+	    gtk_box_pack_start (GTK_BOX (pane), table, expandable, TRUE, 0);
 	    table = gtk_table_new(arraysize - i, r=TableWidth(option + i), FALSE);
-            top = 0;
+            top = breakType = 0; expandable = FALSE;
         }
         if(!SameRow(&option[i])) {
 	    if(SameRow(&option[i+1])) {
@@ -1247,6 +1321,8 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 	    if(option[i].type == FileName || option[i].type == PathName) w -= 55;
 
             if (option[i].type==TextBox && option[i].value > 80){
+                GtkTextIter iter;
+                expandable = TRUE;
                 textview = gtk_text_view_new();
                 gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(textview), option[i].min & T_WRAP ? GTK_WRAP_WORD : GTK_WRAP_NONE);
 #ifdef TODO_GTK
@@ -1272,12 +1348,15 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
                     /* no label so let textview occupy all columns */
                     Pack(hbox, table, sw, left, left+r, top, GTK_EXPAND);
                 }
+                SetWidgetFont(textview, option[i].font);
                 if ( *(char**)option[i].target != NULL )
                     gtk_text_buffer_set_text (textbuffer, *(char**)option[i].target, -1);
                 else
                     gtk_text_buffer_set_text (textbuffer, "", -1);
                 option[i].handle = (void*)textbuffer;
                 option[i].textValue = (char*)textview;
+                gtk_text_buffer_get_iter_at_offset(textbuffer, &iter, -1);
+                gtk_text_buffer_create_mark(textbuffer, "scrollmark", &iter, FALSE); // permanent mark
 		if(option[i].choice) { // textviews can request a handler for mouse events in the choice field
 		    g_signal_connect(textview, "button-press-event", G_CALLBACK (MemoEvent), (gpointer) &option[i] );
 		    g_signal_connect(textview, "button-release-event", G_CALLBACK (MemoEvent), (gpointer) &option[i] );
@@ -1336,6 +1415,7 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
             option[i].handle = (void *) (label = gtk_label_new(option[i].name));
             /* Left Justify */
             gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+            SetWidgetFont(label, option[i].font);
 	    if(option[i].min & BORDER) {
 		GtkWidget *frame = gtk_frame_new(NULL);
                 gtk_container_add(GTK_CONTAINER(frame), label);
@@ -1350,11 +1430,12 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 		g_signal_connect(label, "button-press-event", G_CALLBACK(MemoEvent), (gpointer) &option[i]);
 		gtk_widget_set_sensitive(label, TRUE);
 	    }
-            Pack(hbox, table, label, left, left+2, top, 0);
+            Pack(hbox, table, label, left, left+3, top, 0);
 	    break;
           case SaveButton:
           case Button:
             button = gtk_button_new_with_label (option[i].name);
+            SetWidgetFont(gtk_bin_get_child(GTK_BIN(button)), option[i].font);
 
             /* set button color on view board dialog */
             if(option[i].choice && ((char*)option[i].choice)[0] == '#' && !currentCps) {
@@ -1364,10 +1445,14 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 
             /* set button color on new variant dialog */
             if(option[i].textValue) {
+                static char *b = "Bold";
                 gdk_color_parse( option[i].textValue, &color );
                 gtk_widget_modify_bg ( GTK_WIDGET(button), GTK_STATE_NORMAL, &color );
                 gtk_widget_set_sensitive(button, option[i].value >= 0 && (appData.noChessProgram
 					 || strstr(first.variants, VariantName(option[i].value))));
+                if(engineVariant[100] ? !strcmp(engineVariant+100, option[i].name) : 
+                   gameInfo.variant ? option[i].value == gameInfo.variant : !strcmp(option[i].name, "Normal"))
+                    SetWidgetFont(gtk_bin_get_child(GTK_BIN(button)), &b);
             }
 
             Pack(hbox, table, button, left, left+1, top, 0);
@@ -1416,6 +1501,7 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
                 GtkListStore *store;
 
                 option[i].handle = (void *) (list = gtk_tree_view_new());
+                SetWidgetFont(option[i].handle, option[i].font);
                 gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(list), FALSE);
                 renderer = gtk_cell_renderer_text_new();
                 column = gtk_tree_view_column_new_with_attributes("List Items", renderer, "text", 0, NULL);
@@ -1438,11 +1524,12 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 
                 /* never has label, so let listbox occupy all columns */
                 Pack(hbox, table, sw, left, left+r, top, GTK_EXPAND);
+                expandable = TRUE;
             }
 	    break;
 	  case Graph:
 	    option[i].handle = (void*) (graph = gtk_drawing_area_new());
-//            gtk_widget_set_size_request(graph, option[i].max, option[i].value);
+            gtk_widget_set_size_request(graph, option[i].max, option[i].value);
 	    if(0){ GtkAllocation a;
 		a.x = 0; a.y = 0; a.width = option[i].max, a.height = option[i].value;
 		gtk_widget_set_allocation(graph, &a);
@@ -1452,6 +1539,7 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
             g_signal_connect (graph, "button-press-event", G_CALLBACK (GraphEventProc), (gpointer) &option[i]);
             g_signal_connect (graph, "button-release-event", G_CALLBACK (GraphEventProc), (gpointer) &option[i]);
             g_signal_connect (graph, "motion-notify-event", G_CALLBACK (GraphEventProc), (gpointer) &option[i]);
+            g_signal_connect (graph, "scroll-event", G_CALLBACK (GraphEventProc), (gpointer) &option[i]);
 	    if(option[i].min & FIX_H) { // logo
 		GtkWidget *frame = gtk_aspect_frame_new(NULL, 0.5, 0.5, option[i].max/(float)option[i].value, FALSE);
 		gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
@@ -1459,6 +1547,7 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 		graph = frame;
 	    }
             Pack(hbox, table, graph, left, left+r, top, GTK_EXPAND);
+            expandable = TRUE;
 
 #ifdef TODO_GTK
 	    if(option[i].min & SAME_ROW) last = forelast, forelast = lastrow;
@@ -1473,6 +1562,9 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 	  case DropDown:
 	    top--;
 	    msg = _(option[i].name); // write name on the menu button
+#ifndef OSXAPP
+	    if(tinyLayout) { strcpy(def, msg); def[tinyLayout] = NULLCHAR; msg = def; } // clip menu text to keep menu bar small
+#endif
 //	    XtSetArg(args[j], XtNmenuName, XtNewString(option[i].name));  j++;
 //	    XtSetArg(args[j], XtNlabel, msg);  j++;
 	    option[i].handle = (void*)
@@ -1498,7 +1590,7 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 	    break;
 	  case BarEnd:
 	    top--;
-#ifndef __APPLE__
+#ifndef OSXAPP
             gtk_table_attach(GTK_TABLE(table), menuBar, left, left+r, top, top+1, GTK_FILL | GTK_EXPAND, GTK_FILL, 2, 1);
 
 	    if(option[i].target) ((ButtonCallback*)option[i].target)(boxStart); // callback that can make sizing decisions
@@ -1511,7 +1603,7 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 		gtkosx_application_set_menu_bar(theApp, GTK_MENU_SHELL(menuBar));
 		gtkosx_application_insert_app_menu_item(theApp, GTK_MENU_ITEM(helpMenu[8].handle), 0); // hack
 		gtkosx_application_sync_menubar(theApp);
-	    } 
+	    }
 #endif
 	    break;
 	  case BoxEnd:
@@ -1521,8 +1613,8 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 	    if(option[i].target) ((ButtonCallback*)option[i].target)(boxStart); // callback that can make sizing decisions
 	    break;
 	  case Break:
-            breakType = option[i].min & SAME_ROW;
-	    top = height; // force next option to start in a new table
+            breakType = option[i].min & SAME_ROW | BORDER; // kludge to flag we must break
+	    option[i].handle = table;
             break;
 
 	  case PopUp:
@@ -1534,10 +1626,15 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 	}
     }
 
+    gtk_table_resize(GTK_TABLE(table), top+1, r);
+    if(dlgNr == BoardWindow && appData.fixedSize) { // inhibit sizing
+	GtkWidget *h = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (h), table, TRUE, FALSE, 2);
+	table = h;
+    }
     if(pane)
-	gtk_box_pack_start (GTK_BOX (pane), table, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (pane), table, expandable, TRUE, 0);
     else
-        gtk_table_resize(GTK_TABLE(table), top+1, r),
 	gtk_box_pack_start (GTK_BOX (/*GTK_DIALOG (dialog)->vbox*/box), table, TRUE, TRUE, 0);
 
     option[i].handle = (void *) table; // remember last table in EndMark handle (for hiding Engine-Output pane).
@@ -1574,6 +1671,9 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 	gtk_window_resize(GTK_WINDOW(dialog), wp[dlgNr]->width, wp[dlgNr]->height);
     }
 
+    for(i=0; option[i].type != EndMark; i++) if(option[i].type == Graph)
+	gtk_widget_set_size_request(option[i].handle, -1, -1); // remove size requests after realization, so user can shrink
+
     return 1; // tells caller he must do initialization (e.g. add specific event handlers)
 }
 
@@ -1597,8 +1697,8 @@ SendTextCB (Widget w, XtPointer client_data, Atom *selection,
 void
 SendText (int n)
 {
-#ifdef TODO_GTK
     char *p = (char*) textOptions[n].choice;
+#ifdef TODO_GTK
     if(strstr(p, "$name")) {
 	XtGetSelectionValue(menuBarWidget,
 	  XA_PRIMARY, XA_STRING,
@@ -1606,8 +1706,9 @@ SendText (int n)
 	  (XtPointer) (intptr_t) n, /* client_data passed to PastePositionCB */
 	  CurrentTime
 	);
-    } else SendString(p);
+    } else
 #endif
+    SendString(p);
 }
 
 void
@@ -1618,7 +1719,7 @@ SetInsertPos (Option *opt, int pos)
 }
 
 void
-HardSetFocus (Option *opt)
+HardSetFocus (Option *opt, DialogClass dlg)
 {
-    FocusOnWidget(opt, 0); // second arg not used in GDK
+    FocusOnWidget(opt, dlg);
 }
